@@ -1,9 +1,12 @@
 toOrg <- function(x, ... )
     UseMethod("toOrg")
 
+
+
 toOrg.data.frame <- function(x, row.names = NULL, ...) {
     is.f <- unlist(lapply(x, function(i) inherits(i, "factor") ||
-                                         inherits(i, "Date")))
+                                         inherits(i, "Date") ||
+                                         inherits(i, "POSIXt")))
     if (any(is.f)) {
         is.f <- which(is.f)
         for (i in seq_along(is.f))
@@ -53,6 +56,8 @@ toOrg.data.frame <- function(x, row.names = NULL, ...) {
     res
 }
 
+
+
 toOrg.Date <- function(x, inactive = FALSE,...) {
     res <- if (inactive)
                strftime(x, "[%Y-%m-%d %a]")
@@ -62,6 +67,8 @@ toOrg.Date <- function(x, inactive = FALSE,...) {
     res
 }
 
+
+
 toOrg.POSIXt <- function(x, inactive = FALSE,...) {
     res <- if (inactive)
                strftime(x, "[%Y-%m-%d %a %H:%M:%S]")
@@ -71,17 +78,26 @@ toOrg.POSIXt <- function(x, inactive = FALSE,...) {
     res
 }
 
+
+
 print.org <- function(x, ...) {
     cat(x, sep = "\n")
     invisible(x)
 }
 
+
+
 readOrg <-function (file, header = TRUE,
-                    dec = ".", comment.char = "",
-                    encoding = "", strip.white = TRUE,
+                    dec = ".",
+                    comment.char = "",
+                    encoding = "",
+                    strip.white = TRUE,
                     stringsAsFactors = FALSE,
                     table.name = NULL, text,
-                    table.missing = NULL, ...) {
+                    table.missing = NULL, ...,
+                    strip.format = TRUE,
+                    strip.horiz.rules = TRUE,
+                    collapse.header = FALSE) {
 
     if (missing(file) && !missing(text)) {
         txt <- text
@@ -109,33 +125,88 @@ readOrg <-function (file, header = TRUE,
         end <- grep("^ *[^|]|^\\s*$", txt[start:length(txt)], perl = TRUE)
         if (!length(end))
             end <- length(txt) else end <- start + min(end) - 2L
-        ## end <- min(end[end > start]) - 1L
+        txt <- txt[start:end]
+
+    } else {
+        start <- grep("^\\s*[|]", txt)
+        if (length(start))
+            start <- min(start)
+        end <- grep("^ *[^|]|^\\s*$", txt[start:length(txt)], perl = TRUE)
+        if (!length(end))
+            end <- length(txt) else end <- start + min(end) - 2L
         txt <- txt[start:end]
     }
 
-    sep <- 0
-    if (header) {
-        head <- txt[1:10]
-        line <- min(grep("^ *\\| [^<]", head)) ## ignore format instructions, such as '<10>'
-        sep <- min(grep("^\\s*\\|-", head, perl = TRUE))
-        headers <- trim(strsplit(head[line], "|",
-                                    fixed = TRUE)[[1]][-1])
-    }
-    if (sep > 0)
-        txt <- txt[-(1:sep)]
+    if (collapse.header)
+        header <- TRUE
 
-    ## drop horizontal lines |--------------|
-    txt <- txt[!grepl("^\\s*\\|-", txt, perl = TRUE)]
+    ## DROP FORMAT  <lrc> <N> <lrcN>
+    ## FIXME use more specific rx
+    if (strip.format)
+        txt <- txt[!(grepl("^\\s*[|]\\s*<[0-9lcr]+>[0-9<>rlc| ]*$",
+                           txt, perl = TRUE) |
+                     grepl("^\\s*[|]\\s*/[<>| ]*$",
+                           txt, perl = TRUE))]
+
+
+    ## HORIZONTAL RULES
+    hr <- grepl("^\\s*\\|-", txt, perl = TRUE)
+
+    ##   !collapse.header && strip.horiz.rules
+    ##   ==> remove all rules
+    if (!collapse.header && strip.horiz.rules)
+        txt <- txt[!hr]
+
+
+    ##    collapse.header && strip.horiz.rules
+    ##   ==> remove only leading rules
+    if ( collapse.header && strip.horiz.rules) {
+        cr <- which(!hr) ## content rows
+        if (!length(cr))
+            txt <- character(0)
+        else if (min(cr) > 1)
+            txt <- txt[-seq_len(min(cr) - 1)]
+    }
+
+    if (!strip.horiz.rules) {
+        txt[hr] <- gsub("[+]", "|", txt[hr])
+    }
+
+    if (!header) {
+        h <- 0
+
+    } else {
+
+        if (collapse.header) {
+            h <- grep("^\\s*\\|-", txt, perl = TRUE)
+            if (length(h))
+                h <- min(h) - 1 else h <- 1
+        } else
+            h <- 1
+
+        headers <- txt[ seq_len(h)]
+        txt <-     txt[-seq_len(h)    ]
+
+        headers <- strsplit(headers, "|", fixed = TRUE)
+        headers <- lapply(headers, trimws)
+        headers <- do.call(paste, headers)[-1L]
+
+        if (collapse.header && strip.horiz.rules)
+            ## drop remaining horizontal lines
+            txt <- txt[!grepl("^\\s*\\|-", txt, perl = TRUE)]
+    }
 
     if (length(txt) && any(txt != "")) {
-        res <- read.csv(textConnection(txt), header = FALSE, sep = "|",
+        res <- read.csv(textConnection(txt),
+                        header = FALSE, sep = "|",
                         dec = dec,
                         stringsAsFactors = stringsAsFactors,
                         fileEncoding = encoding,
                         strip.white = strip.white, ...)
 
         ## drop first and last column
-        res <- res[ , c(-1L, -length(res))]
+        res <- res[, c(-1L, -length(res))]
+
     } else {
         res <- vector("list", length = length(headers))
         for (i in seq_along(res))
@@ -143,12 +214,14 @@ readOrg <-function (file, header = TRUE,
         res <- as.data.frame(res, stringsAsFactors = FALSE)
     }
 
-    if (header)
+    if (header && length(headers))
         colnames(res) <- headers
 
     res
 }
 
+
+
 cleanOrg <- function(file,
                      clock = TRUE,
                      state.changes = TRUE,
@@ -186,7 +259,8 @@ cleanOrg <- function(file,
         txt[-ij]
 }
 
-file <- c("~/org", "~/Trading/Server/server.org")
+
+
 .readOrg <-function (file, header = TRUE,
                      dec = ".", comment.char = "",
                      encoding = "", strip.white = TRUE,
@@ -216,9 +290,11 @@ file <- c("~/org", "~/Trading/Server/server.org")
 
 }
 
-##
-.readOrg1 <- function(file,
-                           TODO.states = c("TODO", "DONE"), ...) {
+
+
+.readOrg1 <-
+function(file,
+         TODO.states = c("TODO", "DONE"), ...) {
 
     txt <- readLines(file, ...)
 
@@ -276,9 +352,3 @@ file <- c("~/org", "~/Trading/Server/server.org")
 
     lapply(ans, .entry)
 }
-
-## file <- "~/org/TODO-list.org"
-## ans <- read_TODO_list(file, encoding = "UTF-8")
-## df <- data.frame(created = .Date(unlist(lapply(ans, `[[`, "created"))),
-##                  title = substr(unlist(lapply(ans, `[[`, "title")), 1, 40))
-
